@@ -1,6 +1,6 @@
 //! A logger that prints all messages with a simple, readable output format.
 //!
-//! Optional features include timestamps and colored output.
+//! Optional features include timestamps, colored output and logging to stderr.
 //!
 //! ```
 //! simple_logger::SimpleLogger::new().env().init().unwrap();
@@ -28,12 +28,19 @@
 //! simple_logger::init_with_level(log::Level::Warn).unwrap();
 //! ```
 
-#[cfg(feature = "chrono")]
-use chrono::Local;
+#![cfg_attr(feature = "nightly", feature(thread_id_value))]
+
 #[cfg(feature = "colored")]
 use colored::*;
 use log::{Level, LevelFilter, Log, Metadata, Record, SetLoggerError};
 use std::collections::HashMap;
+#[cfg(feature = "timestamps")]
+use time::{format_description::FormatItem, OffsetDateTime};
+
+#[cfg(feature = "timestamps")]
+const TIMESTAMP_FORMAT: &[FormatItem] = time::macros::format_description!(
+    "[year]-[month]-[day] [hour]:[minute]:[second],[subsecond digits:3]"
+);
 
 /// Implements [`Log`] and a set of simple builder methods for configuration.
 ///
@@ -50,10 +57,16 @@ pub struct SimpleLogger {
     /// directly gives us the desired log level.
     module_levels: Vec<(String, LevelFilter)>,
 
+    /// Whether to include thread names (and IDs) or not
+    ///
+    /// This field is only available if the `threads` feature is enabled.
+    #[cfg(feature = "threads")]
+    threads: bool,
+
     /// Whether to include timestamps or not
     ///
-    /// This field is only available if the `chrono` feature is enabled.
-    #[cfg(feature = "chrono")]
+    /// This field is only available if the `timestamps` feature is enabled.
+    #[cfg(feature = "timestamps")]
     timestamps: bool,
 
     /// Whether to use color output or not.
@@ -80,7 +93,10 @@ impl SimpleLogger {
             default_level: LevelFilter::Trace,
             module_levels: Vec::new(),
 
-            #[cfg(feature = "chrono")]
+            #[cfg(feature = "threads")]
+            threads: false,
+
+            #[cfg(feature = "timestamps")]
             timestamps: true,
 
             #[cfg(feature = "colored")]
@@ -207,11 +223,22 @@ impl SimpleLogger {
         self
     }
 
+    /// Control whether thread names (and IDs) are printed or not.
+    ///
+    /// This method is only available if the `threads` feature is enabled.
+    /// Thread names are disabled by default.
+    #[must_use = "You must call init() to begin logging"]
+    #[cfg(feature = "threads")]
+    pub fn with_threads(mut self, threads: bool) -> SimpleLogger {
+        self.threads = threads;
+        self
+    }
+
     /// Control whether timestamps are printed or not.
     ///
-    /// This method is only available if the `chrono` feature is enabled.
+    /// This method is only available if the `timestamps` feature is enabled.
     #[must_use = "You must call init() to begin logging"]
-    #[cfg(feature = "chrono")]
+    #[cfg(feature = "timestamps")]
     pub fn with_timestamps(mut self, timestamps: bool) -> SimpleLogger {
         self.timestamps = timestamps;
         self
@@ -303,31 +330,63 @@ impl Log for SimpleLogger {
                 record.module_path().unwrap_or_default()
             };
 
-            #[cfg(feature = "chrono")]
-            if self.timestamps {
-                #[cfg(not(feature = "stderr"))]
-                println!(
-                    "{} {:<5} [{}] {}",
-                    Local::now().format("%Y-%m-%d %H:%M:%S,%3f"),
-                    level_string,
-                    target,
-                    record.args()
-                );
-                #[cfg(feature = "stderr")]
-                eprintln!(
-                    "{} {:<5} [{}] {}",
-                    Local::now().format("%Y-%m-%d %H:%M:%S,%3f"),
-                    level_string,
-                    target,
-                    record.args()
-                );
-                return;
-            }
+            let thread = {
+                #[cfg(feature = "threads")]
+                if self.threads {
+                    let thread = std::thread::current();
+
+                    format!("@{}", {
+                        #[cfg(feature = "nightly")]
+                        {
+                            thread.name().unwrap_or(&thread.id().as_u64().to_string())
+                        }
+
+                        #[cfg(not(feature = "nightly"))]
+                        {
+                            thread.name().unwrap_or("?")
+                        }
+                    })
+                } else {
+                    "".to_string()
+                }
+
+                #[cfg(not(feature = "threads"))]
+                ""
+            };
+
+            let timestamp = {
+                #[cfg(feature = "timestamps")]
+                if self.timestamps {
+                    format!("{} ", OffsetDateTime::now_local().expect(concat!(
+                        "Could not determine the UTC offset on this system. ",
+                        "Possible causes are that the time crate does not implement \"local_offset_at\" ",
+                        "on your system, or that you are running in a multi-threaded environment and ",
+                        "the time crate is returning \"None\" from \"local_offset_at\" to avoid unsafe ",
+                        "behaviour. See the time crate's documentation for more information. ",
+                        "(https://time-rs.github.io/internal-api/time/index.html#feature-flags)"
+                    )).format(&TIMESTAMP_FORMAT).unwrap())
+                } else {
+                    "".to_string()
+                }
+
+                #[cfg(not(feature = "timestamps"))]
+                ""
+            };
+
+            let message = format!(
+                "{}{:<5} [{}{}] {}",
+                timestamp,
+                level_string,
+                target,
+                thread,
+                record.args()
+            );
 
             #[cfg(not(feature = "stderr"))]
-            println!("{:<5} [{}] {}", level_string, target, record.args());
+            println!("{}", message);
+
             #[cfg(feature = "stderr")]
-            eprintln!("{:<5} [{}] {}", level_string, target, record.args());
+            eprintln!("{}", message);
         }
     }
 
@@ -436,7 +495,7 @@ mod test {
     }
 
     #[test]
-    #[cfg(feature = "chrono")]
+    #[cfg(feature = "timestamps")]
     fn test_with_timestamps() {
         let mut builder = SimpleLogger::new();
         assert!(builder.timestamps == true);
